/* Copyright (c) 2010, NICTA 
 * All rights reserved. 
 * 
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations 
 * under the License. 
 * 
 * Author: James Petterson (james.petterson@nicta.com.au)
 */

#ifndef _GENERICDATA_HPP_
#define _GENERICDATA_HPP_

#include <vector>
#include "common.hpp"
#include "sml.hpp"
#include "data.hpp"
#include "configuration.hpp"

using namespace std;

class CGenericData: public CData
{
	public:
		CGenericData(unsigned int start=0, unsigned int nparts=1);
		virtual ~CGenericData();

		vector<double*> features;        /* for each image, image features (dense) */

		vector<int*>    features_idxs;   /* for each image, image features (sparse, indexes) */
		vector<double*> features_vals;   /* for each image, image features (sparse, values) */
		vector<int>     features_nnz;    /* for each image, image features (sparse, nnz) */

		vector<int*>    tags;            /* for each image, image tags */
		int           **images_with_tag; /* for each tag, list of images that contain it */

		int N;                   /* number of images */
		int x_dim;               /* feature dimensionality */
		int vocab_size;          /* size of annotation dictionary */

		virtual bool         bias(void)       const { return false; }
		virtual unsigned int slice_size(void) const { return N; }
		virtual unsigned int size(void)       const { return N; }
		virtual unsigned int dim(void)        const 
		{ 
			/* this function is called only when executing linear-bmrm-predict */
			return x_dim*vocab_size; 
		}
	private:
		CGenericData(const CGenericData&);
};

#endif
