/* Copyright (c) 2010, NICTA 
 * All rights reserved. 
 * 
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations 
 * under the License. 
 * 
 * Author: James Petterson (james.petterson@nicta.com.au)
 */

#ifndef _GENERICLOSS_HPP_
#define _GENERICLOSS_HPP_

#include "common.hpp"
#include "sml.hpp"
#include "data.hpp"
#include "genericdata.hpp"
#include "loss.hpp"
#include <string>
#include "timer.hpp"
#include "math.h"
#include <vector>


/* struct to sort scores */
typedef struct
{
	double score;
	int    id;
} score_t;

enum SCORES { MACRO_FBETA=0, HAMMING_LOSS=1 };

/* Class for image annotation loss. */
class CGenericLoss : public CLoss
{
	public:
		CGenericLoss(int dim, CModel* &model, CGenericData* &data);
		virtual ~CGenericLoss();

		// Interfaces
		void Usage() {}
		void ComputeLoss(double &loss)
		{
			throw CBMRMException("ERROR: not implemented!\n", "CGenericLoss::ComputeLoss()");
		}

		void ComputeLossAndGradient(double &loss, TheMatrix &grad);
		void Predict(CModel *model);
		void Evaluate(CModel *model);

	protected:
		CGenericData* data;
		int  max_sparsity;  /* max number of images per label to consider when searching for the most-violated contraint */
		int  sparse_features;
		int  score_to_optimize;
		double beta_sq; /* beta^2 (for macro-Fbeta) */

		void find_most_violated_constraint(int n, int *ybar, TheMatrix &w, bool only_classify);
		int  is_sparsity_bound(int sparsity, int gt_sparsity, double *image_scores, int *dense_y, double beta_sq);
		double label_loss(int *y, int *ybar, double *precision=NULL, double *recall=NULL);
};

#endif
